package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.gson.Gson;
import com.qmth.distributed.print.business.bean.dto.*;
import com.qmth.distributed.print.business.bean.result.WorkResult;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamCardStatusEnum;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.enums.ReviewStatusEnum;
import com.qmth.distributed.print.business.mapper.ExamTaskMapper;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.util.ExcelUtil;
import com.qmth.distributed.print.business.util.ServletUtil;
import com.qmth.distributed.print.business.util.excel.ExcelError;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 命题任务 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class ExamTaskServiceImpl extends ServiceImpl<ExamTaskMapper, ExamTask> implements ExamTaskService {

    @Autowired
    private ExamTaskTempService examTaskTempService;

    @Autowired
    private BasicExamRuleService basicExamRuleService;

    @Autowired
    private ExamTaskReviewLogService examTaskReviewLogService;

    @Autowired
    private ExamTaskDetailService examTaskDetailService;

    @Autowired
    private BasicAttachmentService basicAttachmentService;

    @Autowired
    private CommonService commonService;

    @Resource
    ExamTaskMapper examTaskMapper;

    @Autowired
    private BasicCourseService basicCourseService;

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private ExamCardService examCardService;

    @Autowired
    private ExamDetailCourseService examDetailCourseService;

    @Override
    public List<ExamTask> listByCourseCode(Long schoolId, String code) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId).eq(ExamTask::getCourseCode, code);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<ExamTaskDto> list(Boolean enable, String status, Long cardRuleId, String courseCode, String paperNumber, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds();
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listPage(page, schoolId, enable, status, cardRuleId, courseCode, paperNumber, startTime, endTime, orgIds);
        return examTaskDtoIPage;
    }

    @Override
    public List<String> listPaperNumber(String param) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        // 查询命题任务中的试卷编号
        QueryWrapper<ExamTask> queryWrapperExamTask = new QueryWrapper<>();
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotBlank(param)) {
            queryWrapperExamTask.lambda().eq(ExamTask::getSchoolId, schoolId)
                    .and(i -> i.like(ExamTask::getCourseCode, param).or().like(ExamTask::getCourseName, param));
        }
        List<ExamTask> examTasks = this.list(queryWrapperExamTask);
        if (examTasks != null && examTasks.size() > 0) {
            list = examTasks.stream().map(ExamTask::getPaperNumber).distinct().collect(Collectors.toList());
        }

        // 查询考务-科目中的试卷编号
        QueryWrapper<ExamDetailCourse> queryWrapperExamDetailCourse = new QueryWrapper<>();
        List<String> list2 = new ArrayList<>();
        if (StringUtils.isNotBlank(param)) {
            queryWrapperExamDetailCourse.lambda().eq(ExamDetailCourse::getSchoolId, schoolId)
                    .and(i -> i.like(ExamDetailCourse::getCourseCode, param).or().like(ExamDetailCourse::getCourseName, param));
        }
        List<ExamDetailCourse> ExamDetailCourses = examDetailCourseService.list(queryWrapperExamDetailCourse);
        if (ExamDetailCourses != null && ExamDetailCourses.size() > 0) {
            list2 = ExamDetailCourses.stream().map(ExamDetailCourse::getPaperNumber).distinct().collect(Collectors.toList());
        }
        list.addAll(list2);
        return list.stream().distinct().collect(Collectors.toList());
    }

    @Override
    public List<BlurryUserDto> listUser(String param) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.listUser(schoolId, param);
    }

    @Override
    public boolean assignUser(ExamTask examTask) {
        if (examTask.getId() == null) {
            throw ExceptionResultEnum.ERROR.exception("参数有误");
        }

        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        ExamTask task = this.getById(examTask.getId());
        // 指派命题老师
        if (task.getStatus().name().equals(ExamStatusEnum.NEW.name())) {
            if (!task.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("命题任务已禁用，不能指派命题老师");
            }
            updateWrapper.lambda().set(ExamTask::getUserId, examTask.getUserId()).set(ExamTask::getStatus, ExamStatusEnum.READY).eq(ExamTask::getId, examTask.getId());
        }

        // 更改命题老师
        if (task.getStatus().name().equals(ExamStatusEnum.READY.name())) {
            if (task.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("命题任务禁用后，才能更改命题老师");
            }
            updateWrapper.lambda().set(ExamTask::getUserId, examTask.getUserId()).eq(ExamTask::getId, examTask.getId());
        }

        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean enable(ExamTask examTask) {
        if (examTask.getId() == null) {
            throw ExceptionResultEnum.ERROR.exception("参数有误");
        }

        ExamTask task = this.getById(examTask.getId());
        if (task.getStatus().name().equals(ExamStatusEnum.FINISH)) {
            throw ExceptionResultEnum.ERROR.exception("命题任务状态为已完成，不能启用/禁用操作");
        }
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        // 启用时，后续状态清空
        if (examTask.getEnable()) {
            updateWrapper.lambda().set(ExamTask::getEnable, examTask.getEnable()) // 修改启用状态
                    .set(ExamTask::getReviewStatus, null) // 清空审核状态
                    .set(ExamTask::getStatus, ExamStatusEnum.STAGE) // 更新为命题中
                    .eq(ExamTask::getId, examTask.getId());
        } else {
            updateWrapper.lambda().set(ExamTask::getEnable, examTask.getEnable()).eq(ExamTask::getId, examTask.getId());
        }

        // 解除题卡绑定
        if (task.getEnable()) {
            examTaskDetailService.resetCardId(examTask.getId());
        }
        return this.update(updateWrapper);
    }

    @Override
    public boolean saveExamTask(ExamTask examTask) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        examTask.setSchoolId(schoolId);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        // todo paperNumber生成问题， 必填字段校验
        QueryWrapper<BasicExamRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicExamRule::getSchoolId, examTask.getSchoolId());
        BasicExamRule basicExamRule = basicExamRuleService.getOne(queryWrapper);
        if (basicExamRule == null) {
            throw ExceptionResultEnum.ERROR.exception("通用规则未设置");
        }
        if (examTask.getUserId() == null) {
            examTask.setStatus(ExamStatusEnum.NEW);
        } else {
            examTask.setStatus(ExamStatusEnum.READY);
        }
        examTask.setEnable(true);
        examTask.setCreateId(sysUser.getId());
        examTask.setCreateTime(System.currentTimeMillis());
        examTask.setReview(basicExamRule.getReview());
        return this.save(examTask);
    }

    @Override
    public ExamTaskImportDto importFile(MultipartFile file) throws IOException, NoSuchFieldException {
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(file.getInputStream(), Lists.newArrayList(ExamTaskTempDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
            List<ExcelError> excelErrorTemp = new ArrayList<>();
            // 只允许导入一个sheet
            if (finalExcelList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("excel中只允许有一个sheet");
            }
            for (int i = 0; i < finalExcelList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                List<Object> examTaskTempList = excelMap.get(i);
                for (int y = 0; y < examTaskTempList.size(); y++) {
                    ExamTaskTempDto userImportDto = (ExamTaskTempDto) examTaskTempList.get(y);
                    if (StringUtils.isBlank(userImportDto.getCourseCode())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程代码]必填"));
                    }
                    if (StringUtils.isBlank(userImportDto.getCourseName())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程名称]必填"));
                    }

                    // 课程是否存在并且与名称对应
                    QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
                    queryWrapper.lambda().eq(BasicCourse::getCode, userImportDto.getCourseCode());
                    BasicCourse course = basicCourseService.getOne(queryWrapper);
                    if (course == null) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程代码]在系统中不存在"));
                    } else if (!userImportDto.getCourseName().equals(course.getName())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程名称]与系统中不匹配"));
                    }

                    if (StringUtils.isBlank(userImportDto.getPaperNumber())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[试卷编号]必填"));
                    }
                }
            }
            if (excelErrorTemp.size() > 0) {
                throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(excelErrorTemp));
            }
            return finalExcelList;
        });
        return executeImportExamTaskBatch(finalList);
    }

    @Transactional
    ExamTaskImportDto executeImportExamTaskBatch(List<LinkedMultiValueMap<Integer, Object>> finalList) {
        // 学校ID
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        // 用户
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 批次号
        Gson gson = new Gson();
        String batchNo = String.valueOf(System.nanoTime());
        ExamTaskImportDto examTaskImportDto = new ExamTaskImportDto();
        examTaskImportDto.setBatchNo(batchNo);
        List<ExamTaskDto> tasks = new ArrayList<>();
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> examTaskTempList = excelMap.get(i);
            for (int y = 0; y < examTaskTempList.size(); y++) {
                ExamTaskTempDto examTaskTempDto = (ExamTaskTempDto) examTaskTempList.get(y);
                ExamTaskTemp examTaskTemp = gson.fromJson(gson.toJson(examTaskTempDto), ExamTaskTemp.class);
                examTaskTemp.setId(SystemConstant.getDbUuid());
                examTaskTemp.setBatchNo(batchNo);
                examTaskTemp.setSchoolId(schoolId);
                examTaskTemp.setCreateId(sysUser.getId());
                examTaskTemp.setCreateTime(System.currentTimeMillis());
                //保存
                examTaskTempService.save(examTaskTemp);

                ExamTaskDto examTaskDto = new ExamTaskDto();
                examTaskDto.setCourseCode(examTaskTemp.getCourseCode());
                examTaskDto.setCourseName(examTaskTemp.getCourseName());
                examTaskDto.setPaperNumber(examTaskTemp.getPaperNumber());
                examTaskDto.setSpecialty(examTaskTemp.getSpecialty());
                // 校验命题老师
                if (StringUtils.isBlank(examTaskTemp.getUserAccount())) {
                    examTaskDto.setUsers(listUsers(examTaskTemp.getCourseCode(), null));
                } else {
                    QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
                    queryWrapper.lambda().eq(SysUser::getLoginName, examTaskTemp.getUserAccount());
                    SysUser sysUser1 = sysUserService.getOne(queryWrapper);
                    if (sysUser1 == null) {
                        examTaskDto.setUsers(listUsers(examTaskTemp.getCourseCode(), null));
                    } else {
                        examTaskDto.setUsers(listUsers(examTaskTemp.getCourseCode(), String.valueOf(sysUser1.getId())));
                    }
                }
                tasks.add(examTaskDto);
            }
        }
        examTaskImportDto.setTasks(tasks);
        return examTaskImportDto;
    }

    private List<BlurryUserDto> listUsers(String courseCode, String userId) {
        return sysUserService.listUser(courseCode, userId);
    }

    @Override
    public boolean saveBatch(ExamTask task) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        task.setSchoolId(schoolId);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        if (StringUtils.isBlank(task.getBatchNo())) {
            throw ExceptionResultEnum.ERROR.exception("batchNo不能为空");
        }
        QueryWrapper<ExamTaskTemp> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskTemp::getBatchNo, task.getBatchNo());
        List<ExamTaskTemp> examTaskTemps = examTaskTempService.list(queryWrapper);
        if (examTaskTemps.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("没有可保存数据");
        }
        List<ExamTask> list = new ArrayList<>();
        for (ExamTaskTemp examTaskTemp : examTaskTemps) {
            ExamTask examTask = new ExamTask();
            examTask.setSchoolId(task.getSchoolId());
            examTask.setCourseCode(examTaskTemp.getCourseCode());
            examTask.setCourseName(examTaskTemp.getCourseName());
            examTask.setSpecialty(examTaskTemp.getSpecialty());
            examTask.setPaperNumber(examTaskTemp.getPaperNumber());
            examTask.setStartTime(task.getStartTime());
            examTask.setEndTime(task.getEndTime());
            examTask.setEnable(true);
            examTask.setCardRuleId(task.getCardRuleId());
            examTask.setBatchNo(task.getBatchNo());
            examTask.setCreateId(sysUser.getId());
            examTask.setCreateTime(System.currentTimeMillis());
            String userId = task.getUsers().stream().map(m -> {
                if (examTaskTemp.getCourseCode().equals(m.get("courseCode"))) {
                    return m.get("userId");
                }
                return "";
            }).findFirst().get();
            if (StringUtils.isNotBlank(userId)) {
                examTask.setUserId(Long.valueOf(userId));
                examTask.setStatus(ExamStatusEnum.READY);
            } else {
                examTask.setStatus(ExamStatusEnum.NEW);
            }
            list.add(examTask);
        }
        return this.saveBatch(list);
    }

    @Override
    public IPage<ExamTaskDto> listTaskApply(String auditStatus, String reviewStatus, Long cardRuleId, String courseCode, String paperNumber, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        String[] strings = new String[]{ExamStatusEnum.NEW.name(), ExamStatusEnum.SUBMIT.name(), ExamStatusEnum.FINISH.name()};
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listTaskApply(page, schoolId, auditStatus, reviewStatus, cardRuleId, courseCode, paperNumber, startTime, endTime, strings, sysUser.getId());
        return examTaskDtoIPage;
    }

    @Override
    public IPage<ExamTaskDto> listTaskReviewUnaudited(String courseCode, String paperNumber, Long userId, Long cardRuleId, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds();
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listTaskReviewUnaudited(page, schoolId, courseCode, paperNumber, userId, cardRuleId, startTime, endTime, orgIds);
        return examTaskDtoIPage;
    }

    @Override
    public IPage<ExamTaskDto> listTaskReviewAudited(String reviewStatus, String courseCode, String paperNumber, Long userId, Long cardRuleId, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds();
        Page<ExamTaskDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDto> examTaskDtoIPage = this.baseMapper.listTaskReviewAudited(page, schoolId, reviewStatus, courseCode, paperNumber, userId, cardRuleId, startTime, endTime, orgIds);
        return examTaskDtoIPage;
    }

    @Transactional
    @Override
    public boolean taskReviewSave(ExamTaskReviewLog taskReviewLog) throws IOException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        taskReviewLog.setOperateId(sysUser.getId());
        taskReviewLog.setOperateTime(System.currentTimeMillis());
        // 审核日志
        examTaskReviewLogService.save(taskReviewLog);

        // 更新命题任务状态
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        ExamStatusEnum statusEnum;
        if (taskReviewLog.getReviewStatus().name().equals(ReviewStatusEnum.PASS.name())) {
            statusEnum = ExamStatusEnum.FINISH;
            // 校验是否可以提交打印状态
            ExamTask examTask = this.getById(taskReviewLog.getExamTaskId());
            commonService.checkData(examTask.getSchoolId(), examTask.getCourseCode(), examTask.getPaperNumber(),sysUser);
        } else {
            statusEnum = ExamStatusEnum.STAGE;
        }
        updateWrapper.lambda().set(ExamTask::getStatus, statusEnum).set(ExamTask::getReviewStatus, taskReviewLog.getReviewStatus()).eq(ExamTask::getId, taskReviewLog.getExamTaskId());
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean taskReviewSaveBatch(ExamTaskReviewLog taskReviewLog) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        ExamStatusEnum statusEnum;
        if (taskReviewLog.getReviewStatus().name().equals(ReviewStatusEnum.PASS.name())) {
            statusEnum = ExamStatusEnum.FINISH;
        } else {
            statusEnum = ExamStatusEnum.STAGE;
        }
        for (Long examTaskId : taskReviewLog.getExamTaskIds()) {
            // 审核日志
            taskReviewLog.setOperateId(sysUser.getId());
            taskReviewLog.setOperateTime(System.currentTimeMillis());
            taskReviewLog.setExamTaskId(examTaskId);
            examTaskReviewLogService.save(taskReviewLog);

            // 更新命题任务状态
            UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamTask::getStatus, statusEnum).set(ExamTask::getReviewStatus, taskReviewLog.getReviewStatus()).eq(ExamTask::getId, taskReviewLog.getExamTaskId());
            this.update(updateWrapper);
        }
        return true;
    }

    @Override
    public IPage<ExamTaskDetailDto> listTaskPaper(String courseCode, String paperNumber, Long startTime, Long endTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = commonService.listSubOrgIds();
        Page<ExamTaskDetailDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamTaskDetailDto> examTaskDtoIPage = this.baseMapper.listTaskPaper(page, schoolId, courseCode, paperNumber, startTime, endTime, orgIds);
        return examTaskDtoIPage;
    }

    @Override
    public List<RelatePaperDto> listPaperTypes(String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        List<RelatePaperDto> list = this.baseMapper.listPaperNumbers(schoolId, courseCode);
        for (RelatePaperDto relatePaperDto : list) {
            QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, relatePaperDto.getId()).eq(ExamTaskDetail::getEnable, true);
            ExamTaskDetail examTaskDetail = this.examTaskDetailService.getOne(queryWrapper);
            if (examTaskDetail != null) {
                String unexposedPaperType = examTaskDetail.getUnexposedPaperType();
                relatePaperDto.setPaperTypes(StringUtils.isBlank(unexposedPaperType) ? null : Arrays.asList(unexposedPaperType.split("/")));
            }
        }
        return list;
    }

    @Transactional
    @Override
    public boolean saveExamTaskDetail(ExamTaskDetail examTaskDetail) throws IOException {
        if (examTaskDetail.getExamTaskId() == null) {
            throw ExceptionResultEnum.ERROR.exception("命题任务ID不能为空");
        }
        if (StringUtils.isBlank(examTaskDetail.getOperateType())) {
            throw ExceptionResultEnum.ERROR.exception("保存类型不能为空");
        }
        // 提交时，校验
        if (ExamStatusEnum.SUBMIT.name().equals(examTaskDetail.getOperateType())) {
            if (StringUtils.isBlank(examTaskDetail.getPaperType())) {
                throw ExceptionResultEnum.ERROR.exception("试卷类型不能为空");
            }
            if (StringUtils.isBlank(examTaskDetail.getPaperAttachmentIds())) {
                throw ExceptionResultEnum.ERROR.exception("试卷ID不能为空");
            }
            if (examTaskDetail.getCardId() == null) {
                throw ExceptionResultEnum.ERROR.exception("题卡ID不能为空");
            }
            if (StringUtils.isBlank(examTaskDetail.getPaperConfirmAttachmentIds())) {
                throw ExceptionResultEnum.ERROR.exception("入库审核ID不能为空");
            }

            // 校验题卡是否提交
            ExamCard examCard = examCardService.getById(examTaskDetail.getCardId());
            if(!ExamCardStatusEnum.SUBMIT.name().equals(examCard.getStatus().name())){
                throw ExceptionResultEnum.ERROR.exception("请先提交题卡");
            }
        }

        // 更新examTask状态status
        ExamTask examTask = this.getById(examTaskDetail.getExamTaskId());
        if (examTask.getStatus().name().equals(ExamStatusEnum.SUBMIT.name()) && !examTask.getReview()) {
            examTask.setStatus(ExamStatusEnum.FINISH);
            // 校验是否可以提交打印状态
            commonService.checkData(examTask.getSchoolId(), examTask.getCourseCode(), examTask.getPaperNumber(),(SysUser) ServletUtil.getRequestUser());
        }

        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamTask::getStatus, ExamStatusEnum.valueOf(examTaskDetail.getOperateType())).set(ExamTask::getReviewStatus, null).eq(ExamTask::getId, examTask.getId());
        this.update(updateWrapper);

        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskDetail.getExamTaskId());
        ExamTaskDetail detail = examTaskDetailService.getOne(queryWrapper);
        if (detail != null) {
            examTaskDetail.setId(detail.getId());
            // 已曝光试卷和未曝光试卷赋值
            examTaskDetail.setExposedPaperType(detail.getExposedPaperType());
            examTaskDetail.setUnexposedPaperType(detail.getUnexposedPaperType());
        } else {
            // 已曝光试卷和未曝光试卷赋值（新增时，已曝光试卷为null，未曝光试卷为paper_type）
            examTaskDetail.setUnexposedPaperType(examTaskDetail.getPaperType());
        }

        return examTaskDetailService.saveOrUpdate(examTaskDetail);
    }

    @Override
    public boolean status(ExamTask examTask) {
        if (examTask.getId() == null) {
            throw ExceptionResultEnum.ERROR.exception("参数有误");
        }
        ExamTask task = this.getById(examTask.getId());
        UpdateWrapper<ExamTask> updateWrapper = new UpdateWrapper<>();
        // 撤回
        if (examTask.getStatus().name().equals(ExamStatusEnum.CANCEL.name())) {
            /*if (!task.getStatus().name().equals(ExamStatusEnum.FINISH.name())) {
                throw ExceptionResultEnum.ERROR.exception("命题任务状态为已完成，才能撤回");
            }*/
            updateWrapper.lambda().set(ExamTask::getStatus, examTask.getStatus()).eq(ExamTask::getId, examTask.getId());
            return this.update(updateWrapper);
        }
        return false;
    }

    @Override
    public ExamTaskDetailCardDto applyGetOne(Long examTaskId) {
        ExamTaskDetailCardDto examTaskDetailDto = this.baseMapper.applyGetOne(examTaskId);
        return examTaskDetailDto;
    }

    /**
     * 查询我的工作台
     *
     * @param iPage
     * @param userId
     * @param schoolId
     * @param status
     * @return
     */
    @Override
    public IPage<WorkResult> queryByMyWork(IPage<Map> iPage, Long userId, Long schoolId, String... status) {
        return examTaskMapper.queryByMyWork(iPage, userId, schoolId, status);
    }

    @Override
    public IPage<WorkResult> queryUserByMyWork(IPage<Map> iPage, Long userId, Long schoolId, String... status) {
        return examTaskMapper.queryUserByMyWork(iPage, userId, schoolId, status);
    }

    /**
     * 查询我的工作台-审核待办
     *
     * @param iPage
     * @param userId
     * @param schoolId
     * @param status
     * @return
     */
    @Override
    public IPage<WorkResult> queryByMyWorkSubmit(IPage<Map> iPage, Long userId, Long schoolId, ExamStatusEnum status) {
        return examTaskMapper.queryByMyWorkSubmit(iPage, userId, schoolId, Objects.nonNull(status) ? status.name() : null);
    }

    @Override
    public void paperDownload(HttpServletResponse response, Long examTaskId) {
        String schoolId = ServletUtil.getRequestHeaderSchoolId().toString();
        QueryWrapper<ExamTaskDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTaskDetail::getExamTaskId, examTaskId);
        ExamTaskDetail examTaskDetail = examTaskDetailService.getOne(queryWrapper);
        if (examTaskDetail == null) {
            throw ExceptionResultEnum.ERROR.exception("数据异常");
        }
        String paperAttachmentIds = examTaskDetail.getPaperAttachmentIds();
        List<JSONObject> jsonObjectList = JSONObject.parseArray(paperAttachmentIds, JSONObject.class);
        // 本地保存目录
        String rootPath = SystemConstant.DOWNLOAD_TEMP + File.separator + schoolId + File.separator + examTaskDetail.getId();
        List<File> fileList = new ArrayList<>();
        // 试卷
        for (JSONObject jsonObject : jsonObjectList) {
            Long id = jsonObject.getLongValue("attachmentId");
            BasicAttachment attachment = basicAttachmentService.getById(id);
            if (attachment == null) {
                throw ExceptionResultEnum.ERROR.exception("附件数据异常");
            }
            File file = commonService.copyFile(rootPath, attachment);
            fileList.add(file);
        }

        // 题卡 todo
//        Long cardId = examTaskDetail.getCardId();
//        if (cardId != null){
//
//        }
        if (fileList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("没有可导出文件");
        }

        commonService.downloadFileAndZip(response, rootPath, fileList);
    }

    @Override
    public ExamTask getByCourseCodeAndPaperNumber(Long schoolId, String courseCode, String paperNumber) {
        QueryWrapper<ExamTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamTask::getSchoolId, schoolId)
                .eq(ExamTask::getCourseCode, courseCode)
                .eq(ExamTask::getPaperNumber, paperNumber);
        return this.getOne(queryWrapper);
    }
}
